/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "resource.h"
#include "DlgUpdateBox.h"
#include <ctype.h>
#include "TinyCadView.h"
#include "registry.h"
#include "colour.h"
#include "TinyCad.h"



////// The Update Dialog Box //////

CDlgUpdateBox::CDlgUpdateBox(CWnd* pParentWnd) 
: CDialog(IDD_UPDATE, pParentWnd ) 
{
	//{{AFX_DATA_INIT(CDlgUpdateBox)
	//}}AFX_DATA_INIT
	m_capture = FALSE;
}

void CDlgUpdateBox::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgUpdateBox)
	DDX_Control(pDX, IDC_DELETE, m_Delete);
	DDX_Control(pDX, IDC_UPDATE_LIST, m_list);
	//}}AFX_DATA_MAP
}


CDlgUpdateBox::~CDlgUpdateBox()
{
}

BEGIN_MESSAGE_MAP( CDlgUpdateBox, CDialog )
	//{{AFX_MSG_MAP(CDlgUpdateBox)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_NOTIFY(NM_CLICK, IDC_UPDATE_LIST, OnClickUpdateList)
	ON_NOTIFY(NM_SETFOCUS, IDC_UPDATE_LIST, OnSetfocusUpdateList)
	ON_WM_DRAWITEM()
	//}}AFX_MSG_MAP
	ON_EN_KILLFOCUS(101, OnKillfocusEdit)
	ON_CBN_KILLFOCUS(102, OnKillfocusList)
END_MESSAGE_MAP()


BOOL CDlgUpdateBox::OnInitDialog()
{
  CDialog ::OnInitDialog();

  SetDlgItemText(UPDATE_LIBFILE, NameLength((NewSymbol->GetLibName()),30));
  SetDlgItemText(UPDATE_DESCRIPTION, NewSymbol->description);
  SetDlgItemInt(UPDATE_PPP,NewSymbol->ppp);

  m_index = -1;
  m_Delete.EnableWindow( FALSE );

  m_list.DeleteAllItems();
  m_list.InsertColumn(0,_T("Parameter"), LVCFMT_LEFT, 100);
  m_list.InsertColumn(1,_T("Show?"), LVCFMT_LEFT, 90);
  m_list.InsertColumn(2,_T("Default value"), LVCFMT_LEFT, 140);

  // Now add the name & reference so that they can be
  // switched off
  CSymbolField	f;
  f.field_name = "Reference";
  f.field_type = NewSymbol->ref_type;
  f.field_default = NewSymbol->reference;
  AddSymbolField( f );  

  f.field_name = "Name";
  f.field_type = NewSymbol->name_type;
  f.field_default = NewSymbol->name;
  f.field_default.MakeUpper();
  AddSymbolField( f );  

  for (int i = 0; i < NewSymbol->fields.size(); i ++)
  {
	  AddSymbolField( NewSymbol->fields[i] );
  }

  return TRUE;
}



void CDlgUpdateBox::OnOK()
{
  if (m_capture)
  {
	  EndEdit();
	  return;
  }

  CSymbolField f;

	// Get the reference
	f = GetSymbolField( 0 );
	NewSymbol->reference = f.field_default;
	NewSymbol->ref_type = f.field_type;

	// Get the name
	f = GetSymbolField( 1 );
	NewSymbol->name = f.field_default;
	NewSymbol->name_type = f.field_type;

	// Now write back the rest of the fields
	NewSymbol->fields.erase( NewSymbol->fields.begin(), NewSymbol->fields.end() );
	for (int i = 2; i < m_list.GetItemCount(); i++)
	{
		f = GetSymbolField( i );
		NewSymbol->fields.push_back( f );
	}

  GetDlgItemText(UPDATE_DESCRIPTION, NewSymbol->description );

  NewSymbol->ppp=GetDlgItemInt(UPDATE_PPP);

  // Only end dialog if it is ok
  if (NewSymbol->IsValid())
	EndDialog(IDOK);
  else
	Message(IDS_BADDETAILS,MB_ICONINFORMATION);
}

static const char *field_types[] = 
{
	"Default to show",
	"Default to hidden",
	"Never show"
};

void CDlgUpdateBox::AddSymbolField( CSymbolField &f )
{
	int index = m_list.GetItemCount();

	LVITEM item;
	item.mask = LVIF_TEXT|LVIF_PARAM; 
	item.iItem = index; 
	item.iSubItem = 0; 
	item.state = 0; 
	item.stateMask = 0; 
	item.iImage = 0; 
	item.iIndent = 0;
	item.lParam = index;
	item.pszText = f.field_name.GetBuffer( 256 );
	m_list.InsertItem( &item );
	f.field_name.ReleaseBuffer();; 

	if (f.field_type >= 0 && f.field_type <= 
		sizeof(field_types) / sizeof(const char *))
	{
		item.mask = LVIF_TEXT;
		item.iSubItem = 1; 
		item.pszText = (char *)field_types[f.field_type];
		m_list.SetItem( &item );
	}


	item.iSubItem = 2; 
	item.pszText = f.field_default.GetBuffer( 256 );
	m_list.SetItem( &item );
	f.field_default.ReleaseBuffer();
}


void CDlgUpdateBox::OnAdd() 
{
	// Add a new parameter to the end of the list
	// control
	CSymbolField f;
	f.field_default = "..";
	f.field_name = "Other";
	f.field_type = default_show;
	AddSymbolField( f );
}

void CDlgUpdateBox::OnDelete() 
{
	if (m_index != 0)
	{
		m_list.DeleteItem( m_index );
		m_index = -1;
		m_Delete.EnableWindow( FALSE );
	}
}


void CDlgUpdateBox::OnClickUpdateList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Find out where we clicked...
	const MSG *pMsg = GetCurrentMessage();
	CPoint clk = CPoint( pMsg->pt.x, pMsg->pt.y );

	// If we are already capturing then send this to the
	// capture item...
	if (m_capture)
	{
		CWnd *pwnd;
		if (m_column == 1)
		{
			pwnd = &m_list_control;
		}
		else
		{
			pwnd = &m_edit_control;
		}
		CRect q;
		pwnd->GetWindowRect( q );
		if (q.PtInRect( clk ))
		{
			pwnd->PostMessage( WM_LBUTTONDOWN, 0, MAKELPARAM(clk.x,clk.y) );
			pwnd->PostMessage( WM_LBUTTONUP, 0, MAKELPARAM(clk.x,clk.y) );
			return;
		}
		else
		{
			EndEdit();
		}
	}

	m_list.ScreenToClient(&clk);
	int index = m_list.HitTest(CPoint(5,clk.y));

	// Did we find this item?
	if (index == -1)
		return;

	// Release any old edit dlg
	if (m_capture)
	{
		EndEdit();
	}


	// Select this item
	LVITEM item;
	item.mask = LVIF_STATE; 
	item.iItem = index; 
	item.iSubItem = 0;
	item.state = LVIS_SELECTED | LVIS_FOCUSED; 
	item.stateMask = LVIS_SELECTED | LVIS_FOCUSED; 
	m_list.SetItem( &item );
	m_index = index;

	// If this index is over 2 then we can enable delete...
	m_Delete.EnableWindow( m_index >= 2 );
	

	// Which column did the user click in?
	CRect rect;
	CRect r;
	m_list.GetItemRect( index, &rect, LVIR_BOUNDS );
	r = rect;
	int x = rect.right - 2;
	r.left = x;
	for (int column=2;column>=0;column--)
	{
		int w = m_list.GetColumnWidth(column);
		x -= w;
		r.right = r.left;
		r.left = x;
		
		if (clk.x > x)
		{
			break;
		}
	}

	m_list.ClientToScreen( &r );
	ScreenToClient( &r );
	m_column = column;


	// Get this column's current value
	CString v;
	item.iItem = m_index; 
	item.iSubItem = m_column; 
	item.mask = LVIF_TEXT;
	item.pszText = v.GetBuffer( 256 );
	item.cchTextMax = 256;
	m_list.GetItem( &item );
	v.ReleaseBuffer();


	// Now perform the appropriate action
	switch (column)
	{
	case 0:
	case 2:
		// Now create the edit control
		if (m_index >= 2 || m_column > 0)
		{
			DWORD style = WS_CHILD|WS_VISIBLE|WS_BORDER;
			if (m_index == 1)
			{
				style |= ES_UPPERCASE;
			}

			m_capture = TRUE;
			m_edit_control.Create( style, r, this, 101 );
			m_edit_control.SetFont( GetFont() );
			m_edit_control.SetWindowText( v );
			m_edit_control.SetSel( 0, v.GetLength() );
			m_edit_control.SetFocus();
		}
		break;
	case 1:
		{
			int sel = -1;
			m_capture = TRUE;
			r.bottom = r.bottom + 250;
			m_list_control.Create( WS_CHILD|WS_VISIBLE|WS_BORDER|CBS_DROPDOWNLIST,
				r, this, 102 );
			m_list_control.SetFont( GetFont() );
			
			for (int i = 0; i < sizeof(field_types) / sizeof(const char *); i++)
			{
				m_list_control.AddString( field_types[i] );
				if (v == field_types[i])
				{
					sel = i;
				}
			}
			m_list_control.SetCurSel(sel);
			m_list_control.SetFocus();
		}
		break;
	}

	
	*pResult = 0;
}


void CDlgUpdateBox::OnKillfocusEdit() 
{
	EndEdit();
}

void CDlgUpdateBox::OnKillfocusList() 
{
	// EndEdit();
}

void CDlgUpdateBox::OnSetfocusUpdateList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (!m_capture)
	{
		OnClickUpdateList( pNMHDR, pResult );	
	}

	*pResult = 0;
}


void CDlgUpdateBox::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CDC dc;
	dc.Attach( lpDrawItemStruct->hDC );

	CPen *old_pen;
	CBrush *old_brush;
	CPen select_pen;
	CBrush select_brush;
	COLORREF old_colour;

	if ((lpDrawItemStruct->itemState & ODS_SELECTED) != 0)
	{
		select_pen.CreatePen( PS_SOLID, 1, GetSysColor(COLOR_HIGHLIGHT) );
		select_brush.CreateSolidBrush( GetSysColor(COLOR_HIGHLIGHT) );

		old_pen = dc.SelectObject( &select_pen );
		old_brush = dc.SelectObject( &select_brush );

		old_colour = dc.SetTextColor( GetSysColor(COLOR_HIGHLIGHTTEXT) );
	}
	else
	{
		old_pen = (CPen *)dc.SelectStockObject( WHITE_PEN );
		old_brush = (CBrush *)dc.SelectStockObject( WHITE_BRUSH );
		old_colour = dc.SetTextColor( RGB(0,0,0) );
	}

	dc.Rectangle( &lpDrawItemStruct->rcItem );

	char buffer[256];
	int left = 0;
	int right = lpDrawItemStruct->rcItem.right;
	for (int column=2;column>=0;column--)
	{
		// Create the rectangle to draw into
		left = right - m_list.GetColumnWidth(column);
		CRect r = lpDrawItemStruct->rcItem;
		r.left = left + 2;
		r.right = right;

		// Now draw this item's text
		LVITEM item;
		item.mask = LVIF_TEXT; 
		item.iItem = lpDrawItemStruct->itemID; 
		item.iSubItem = column; 
		item.state = 0; 
		item.stateMask = 0; 
		item.iImage = 0; 
		item.iIndent = 0;
		item.pszText = buffer;
		item.cchTextMax = sizeof( buffer );
		m_list.GetItem( &item );

		dc.DrawText( buffer, &r, DT_LEFT | DT_VCENTER );
			
		right = left;
	}


	dc.SetTextColor( old_colour );
	dc.SelectObject( old_pen );
	dc.SelectObject( old_brush );
	dc.Detach();
}

void CDlgUpdateBox::EndEdit()
{
	// Now end the editing...
	if (m_capture)
	{
		CString v;
		m_capture = FALSE;

		if (m_column == 1)
		{
			m_list_control.GetWindowText( v );
			m_list_control.DestroyWindow();
		}
		else
		{

			m_edit_control.GetWindowText( v );
			m_edit_control.DestroyWindow();
		}

		// .. and re-display...
		LVITEM item;
		item.iItem = m_index; 
		item.iSubItem = m_column; 
		item.mask = LVIF_TEXT;
		item.pszText = v.GetBuffer( 256 );
		m_list.SetItem( &item );
		v.ReleaseBuffer();
	}

}

CSymbolField CDlgUpdateBox::GetSymbolField(int index)
{
	CSymbolField f;

	LVITEM item;
	item.mask = LVIF_TEXT; 
	item.iItem = index; 
	item.iSubItem = 0; 
	item.cchTextMax = 256;
	item.pszText = f.field_name.GetBuffer( 256 );
	m_list.GetItem( &item );
	f.field_name.ReleaseBuffer();

	CString ft;
	item.iSubItem = 1; 
	item.pszText = ft.GetBuffer( 256 );
	m_list.GetItem( &item );
	ft.ReleaseBuffer();

	// Perform conversion...
	for (int i = 0; i <	sizeof(field_types) / sizeof(const char *); i++)
	{
		if (ft == field_types[i])
		{
			f.field_type = static_cast<SymbolFieldType>(i);
		}
	}


	item.iSubItem = 2; 
	item.pszText = f.field_default.GetBuffer( 256 );
	m_list.GetItem( &item );
	f.field_default.ReleaseBuffer();

	return f;
}

