/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __LIBRARY_H__
#define __LIBRARY_H__

#include "Symbol.h"
#include "LibraryStore.h"


////// The Library class which encapsulates each library in use //////
class CLibraryFile : public CLibraryStore
{
	CFile	theMethodsFile;			// The Method's file 
	BOOL	methodsFileOpen;		// Is the method's file & archive open?
	BYTE	revision;				// The revision of the methods file to use
	CString MethodsFileName();		// Return the current methods file name

	// Write the method into either it's own file or the index
	// file.
	void WriteMethod( CStreamFile &ar, CTinyCadDoc &design );

public:
	CLibraryFile();									// The constructor
	virtual ~CLibraryFile();						// The destructor

	virtual void Attach(const char *filename);		// Attach this library to a file
	virtual void Tidy();							// Tidy up the methods file
	
	// Write a symbol to this library
	virtual void Store( CTinyCadSymbolDoc &document );

	// Write back a symbol collection and rebuild methods file
	// (i.e. partial tidy)
	virtual void SaveSymbolCollection( symbolCollection &temp_symbols );

	virtual void OnIdle() { CloseMethodFile(); }

	virtual CStream *GetMethodArchive( CLibraryStoreSymbol *symbol );	// Get the methods file for this library
	void CloseMethodFile();
	BOOL OpenIndexFile(CFile &,int);			// Get the symbols file for this library
};




///// The dialog box for editing libraries //////




// The dialog for editing which libraries are in use
class CDlgLibraryBox : public CDialog 
{

	CListBox *theListBox;

	CLibraryStore *LibSelected();

	virtual void OnOK();
public:
	// Construction
	CDlgLibraryBox(CWnd *pParentWnd);

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDlgLibraryBox)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
public:

// Dialog Data
	//{{AFX_DATA(CDlgLibraryBox)
	enum { IDD = IDD_LIBRARY };
	CStatic	m_position;
	//}}AFX_DATA


	// Generated message map functions
protected:
	//{{AFX_MSG(CDlgLibraryBox)
	afx_msg BOOL OnInitDialog();
	afx_msg void OnAdd();
	afx_msg void OnRemove();
	afx_msg void OnEdit();
	afx_msg void OnTidy();
	afx_msg void OnDblclkAvailable();
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};



// The dialog for renaming a symbol
class CDlgSymbolRename : public CDialog {
	CString *name;

	afx_msg BOOL OnInitDialog();
	afx_msg void OnOK();
	afx_msg void OnCancel();
public:
	CDlgSymbolRename(CString *NewName,CWnd *pParentWnd=NULL) : CDialog( IDD_SYMBOLRENAME, pParentWnd ) { name=NewName; }
};


// Dialog for selecting which device this is
class CDlgWhichDevice : public CDialog {
	int device;
public:
	CDlgWhichDevice(int NewDevice,CWnd *pParentWnd=NULL) : CDialog( IDD_SELECTDEVICE, pParentWnd ) { device = NewDevice; }

	BOOL OnInitDialog();
	void OnOK();
	int GetPart() { return device; }
};


#endif
