/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "tinycad.h"
#include "TinyCadDoc.h"
#include "Revision.h"
#include "Option.h"
#include "Library.h"
#include "diag.h"
#include "colour.h"
#include "registry.h" 
#include "LibraryDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc

IMPLEMENT_DYNCREATE(CLibraryDoc, CDocument)

CLibraryDoc::CLibraryDoc()
{
	m_selected = -1;
	m_pLibrary = NULL;
}

BOOL CLibraryDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CLibraryDoc::~CLibraryDoc()
{
	
}


BEGIN_MESSAGE_MAP(CLibraryDoc, CDocument)
	//{{AFX_MSG_MAP(CLibraryDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc diagnostics

#ifdef _DEBUG
void CLibraryDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLibraryDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc serialization

void CLibraryDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc commands

void CLibraryDoc::setLibrary(CLibraryStore *pLibrary)
{
	// Keep a track of this library
	m_pLibrary = pLibrary;

	// Now set our name accordingly
	SetTitle(pLibrary->m_name);

	// Make a copy of the library to work with
	CLibraryStore::symbolCollection::iterator it = m_pLibrary->Symbols.begin();
	while (it != m_pLibrary->Symbols.end())
	{
		// First, make a copy of the symbols
		m_symbols[it->first] = it->second;
		it ++;
	}

	// Now make a list of the heights of each of the
	// symbols
	determineSizeList();

	// We are no longer selecting an object
	m_selected = -1;

	// .. and make a check of the symbol changes
	SetModifiedFlag( TRUE );
}

void CLibraryDoc::ResetSymbols()
{
	// Erase everything
	m_symbols.erase( m_symbols.begin(), m_symbols.end() );
	m_heights.erase( m_heights.begin(), m_heights.end() );
	m_selected = -1;


	// Now re-load the symbols
	setLibrary( m_pLibrary );

	// Now invalidate this window
	UpdateAllViews( NULL );
}


// Is this document editing a library?
bool CLibraryDoc::IsLibInUse( CLibraryStore *lib )
{
	return lib == m_pLibrary;
}


void CLibraryDoc::determineSizeList()
{
	// Calculate the size of each of the elements
	int current_y = 0;

	TransformSnap s;
	s.SetGridSnap( FALSE );
	CTinyCadDoc doc;

	const int inter_spacing = 20;
	current_y = 0;

	m_heights.erase(m_heights.begin(),m_heights.end());
	CLibraryStore::symbolCollection::iterator it = m_symbols.begin();

	// Make a copy of the library to work with
	while (it != m_symbols.end())
	{
		// Now access the symbol and draw it next to the name
		CPoint p;
		CDesignFileSymbol *pSymbol = (it->second).GetDesignSymbol( &doc );
		drawingCollection method;

		if (pSymbol)
		{
			pSymbol->GetMethod( p, method );
			delete pSymbol;

			// Note: we use p.x because we have rotated the 
			// symbol
			current_y += -p.x + inter_spacing*2;
			m_heights.insert( current_y );
		}

		// Move on to the next symbol
		it ++;
	}
}


CSize CLibraryDoc::GetDocSize()
{

	if (m_heights.size() > 0)
	{
		return CSize(0,*(m_heights.rbegin()));
	}
	else
	{
		return CSize(0,0);
	}
}

void CLibraryDoc::DeleteSymbol(int which)
{
	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = m_symbols.begin();
	for (int i =0; it != m_symbols.end() && i < which; i++)
	{
		it ++;
	}

	// Now remove this symbol and rebuild the lists etc..
	m_symbols.erase( it );

	// .. and adjust our cached information accordingly
	determineSizeList();
}

void CLibraryDoc::RenameSymbol(int which, const char *newName)
{
	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = m_symbols.begin();
	for (int i =0; it != m_symbols.end() && i < which; i++)
	{
		it ++;
	}

	// Now check that we haven't already got a symbol called this
	// name
	CLibraryStore::symbolCollection::iterator itx = m_symbols.begin();
	for (; itx != m_symbols.end(); itx++)
	{
		if ((itx->second).name == newName )
		{
			// We already have a symbol called this...
			Message(IDS_DUPNAME);
			return;
		}
	}
	

	// Perform the change
	if (it != m_symbols.end())
	{
		(it->second).name = newName;
		SetModifiedFlag( TRUE );
	}
}

void CLibraryDoc::EditSymbol(int which)
{
	// Is this a new symbol?
	if (which == -1)
	{
		CLibraryStoreSymbol new_symbol;
		new_symbol.description = "New symbol";
		new_symbol.FilePos = -1;
		new_symbol.lib = m_pLibrary;
		new_symbol.name = "New symbol";
		new_symbol.ppp = 1;
		new_symbol.reference = "U?";

		static_cast<CTinyCadApp*>(AfxGetApp())->editSymbol( m_pLibrary, new_symbol );
	}
	else
	{
		// Determine which symbol we are talking about
		CLibraryStore::symbolCollection::iterator it = m_symbols.begin();
		for (int i =0; it != m_symbols.end() && i < which; i++)
		{
			it ++;
		}

		if (it != m_symbols.end())
		{
			static_cast<CTinyCadApp*>(AfxGetApp())->editSymbol( m_pLibrary, it->second );
		}
	}
}

BOOL CLibraryDoc::SaveModified() 
{
	CString prompt;
	AfxFormatString1(prompt, AFX_IDP_ASK_TO_SAVE, m_pLibrary->m_name );
	switch (AfxMessageBox(prompt, MB_YESNOCANCEL, AFX_IDP_ASK_TO_SAVE))
	{
	case IDCANCEL:
		return FALSE;
	case IDYES:
		m_pLibrary->SaveSymbolCollection( m_symbols );
		return TRUE;
	case IDNO:
		break;
	}

	return TRUE;
}
