/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "startup.h"
#include "TinyCad.h"
#include "TinyCadDoc.h"

#include "EditToolBar.h"
#include "DlgPositionBox.h"

#include "MainFrm.h"

#include "Registry.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CEditToolbar g_EditToolBar;


/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_MOVE()
	ON_WM_CLOSE()
	ON_COMMAND(ID_VIEW_ANNOTATIONTOOLBAR, OnViewAnnotationtoolbar)
	ON_COMMAND(ID_VIEW_EDITTOOLBAR, OnViewEdittoolbar)
	ON_COMMAND(ID_VIEW_FILETOOLBAR, OnViewFiletoolbar)
	ON_COMMAND(ID_VIEW_GROUPTOOLBAR, OnViewGrouptoolbar)
	ON_COMMAND(ID_VIEW_BUSTOOLBAR, OnViewBustoolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_BUSTOOLBAR, OnUpdateViewBustoolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ANNOTATIONTOOLBAR, OnUpdateViewAnnotationtoolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_EDITTOOLBAR, OnUpdateViewEdittoolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FILETOOLBAR, OnUpdateViewFiletoolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GROUPTOOLBAR, OnUpdateViewGrouptoolbar)
	ON_COMMAND(IDM_VIEWSHOWPOS, OnViewshowpos)
	ON_UPDATE_COMMAND_UI(IDM_VIEWSHOWPOS, OnUpdateViewshowpos)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
	delete m_pStartup;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	CRect r(0,0,0,0);
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, r, IDR_MAINFRAME) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	if (!m_wndToolBarDraw.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, r, IDR_DRAWING) ||
		!m_wndToolBarDraw.LoadToolBar(IDR_DRAWING))
	{
		TRACE0("Failed to create toolbar for drawing\n");
		return -1;      // fail to create
	}
	if (!m_wndToolBarDrawBus.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, r, IDR_DRAWBUS) ||
		!m_wndToolBarDrawBus.LoadToolBar(IDR_DRAWBUS))
	{
		TRACE0("Failed to create toolbar for buses\n");
		return -1;      // fail to create
	}
	if (!m_wndToolBarDrawGroup.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, r, IDR_DRAWGROUP) ||
		!m_wndToolBarDrawGroup.LoadToolBar(IDR_DRAWGROUP))
	{
		TRACE0("Failed to create toolbar for group operations\n");
		return -1;      // fail to create
	}

	if (!m_wndToolBarAnnotate.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, r, IDR_ANNOTATE) ||
		!m_wndToolBarAnnotate.LoadToolBar(IDR_ANNOTATE))
	{
		TRACE0("Failed to create toolbar for annotations\n");
		return -1;      // fail to create
	}


	if (!m_wndPositionBox.Create(this,  CDlgPositionBox::IDD, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, CDlgPositionBox::IDD) )
	{
		TRACE0("Failed to create toolbar for annotations\n");
		return -1;      // fail to create
	}


	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_wndToolBarDraw.EnableDocking(CBRS_ALIGN_ANY);
	m_wndToolBarAnnotate.EnableDocking(CBRS_ALIGN_ANY);
	m_wndToolBarDrawBus.EnableDocking(CBRS_ALIGN_ANY);
	m_wndToolBarDrawGroup.EnableDocking(CBRS_ALIGN_ANY);
	m_wndPositionBox.EnableDocking(CBRS_ALIGN_ANY);
	
	EnableDocking(CBRS_ALIGN_ANY);

	DockControlBar(&m_wndToolBar, AFX_IDW_DOCKBAR_TOP );
	RecalcLayout();

	CRect rect;
	m_wndToolBar.GetWindowRect(&rect);
	rect.left=rect.right-2;
	rect.right = rect.right + 40;

	DockControlBar(&m_wndToolBarDraw, AFX_IDW_DOCKBAR_TOP, &rect);
	rect.left++;
	DockControlBar(&m_wndToolBarAnnotate, AFX_IDW_DOCKBAR_TOP, &rect);
	rect.left++;
	DockControlBar(&m_wndToolBarDrawBus, AFX_IDW_DOCKBAR_TOP, &rect);
	rect.left++;
	DockControlBar(&m_wndToolBarDrawGroup, AFX_IDW_DOCKBAR_TOP, &rect);
	rect.left++;
	DockControlBar(&m_wndPositionBox, AFX_IDW_DOCKBAR_TOP, &rect);

	LoadBarState("Settings");

	g_EditToolBar.Create( this );
	GetWindowRect( m_oldRect );


	// Now show the splash screen
	m_pStartup = new CDlgStartUpWindow( this );

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CMDIFrameWnd::PreCreateWindow(cs) )
		return FALSE;


	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CMDIFrameWnd::OnSize(nType, cx, cy);
	
	CRect newRect;
	GetWindowRect( newRect );

	CRect currentRect;
	g_EditToolBar.GetWindowRect( currentRect );
	currentRect -= CPoint( m_oldRect.right - newRect.right, m_oldRect.top - newRect.top );
	g_EditToolBar.MoveWindow( currentRect );

	m_oldRect = newRect;

	((CTinyCadApp*)AfxGetApp())->SetMaximize( IsZoomed() );
}


void CMainFrame::OnMove(int x, int y) 
{
	CMDIFrameWnd::OnMove(x, y);
	
	CRect newRect;
	GetWindowRect( newRect );
	CRect currentRect;
	g_EditToolBar.GetWindowRect( currentRect );
	currentRect -= CPoint( m_oldRect.right - newRect.right, m_oldRect.top - newRect.top );
	g_EditToolBar.MoveWindow( currentRect );
	m_oldRect = newRect;

}



void CMainFrame::OnClose() 
{
	SaveBarState("Settings");
	
	CMDIFrameWnd::OnClose();
}

void CMainFrame::OnViewAnnotationtoolbar() 
{
	ShowControlBar(&m_wndToolBarAnnotate, (m_wndToolBarAnnotate.IsWindowVisible() ==0), FALSE); 
}

void CMainFrame::OnUpdateViewAnnotationtoolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndToolBarAnnotate.IsWindowVisible());
	
}

void CMainFrame::OnViewBustoolbar() 
{
	ShowControlBar(&m_wndToolBarDrawBus, (m_wndToolBarDrawBus.IsWindowVisible() ==0), FALSE); 
	
}

void CMainFrame::OnUpdateViewBustoolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndToolBarDrawBus.IsWindowVisible()); 
	
}

void CMainFrame::OnViewEdittoolbar() 
{
	ShowControlBar(&m_wndToolBarDraw, (m_wndToolBarDraw.IsWindowVisible() ==0), FALSE); 
}

void CMainFrame::OnUpdateViewEdittoolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndToolBarDraw.IsWindowVisible());
	
}


void CMainFrame::OnViewFiletoolbar() 
{
	ShowControlBar(&m_wndToolBar, (m_wndToolBar.IsWindowVisible() ==0), FALSE); 
	
}

void CMainFrame::OnUpdateViewFiletoolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndToolBar.IsWindowVisible());
	
}


void CMainFrame::OnViewGrouptoolbar() 
{
	ShowControlBar(&m_wndToolBarDrawGroup, (m_wndToolBarDrawGroup.IsWindowVisible() ==0), FALSE); 
	
}


void CMainFrame::OnUpdateViewGrouptoolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndToolBarDrawGroup.IsWindowVisible());
	
}


void CMainFrame::OnViewshowpos() 
{
	m_wndPositionBox.ShowWindow( m_wndPositionBox.IsWindowVisible() ? SW_HIDE : SW_SHOW );
}

void CMainFrame::OnUpdateViewshowpos(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndPositionBox.IsWindowVisible());
	
}
