/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __NET_H__
#define __NET_H__

#define NetHashSize	5		// The size of the Net Hash Table
#define NetComment	";"		// The symbol used for comments


class NetHash;


// This is a B-tree which is used in the structures below
class BTree {
protected:
	int key;
	BTree *left,*right;
public:
	void DoAdd(BTree *);		// Add a node to this tree
};


// This structure is for a balanced tree of nodes
class Node : public BTree {
private:
	int Compare(BTree *);		// Are the two nodes comparible?
public:
	int NetList;			// The netlist this node is a member of
	Node *NetPtr;			// The list pointer for the netlist
	CString name;			// The name of this node
	CPoint a;			// The point where this node is on a design
	CDrawingObject *parent;			// The object which generated this node
	CDrawMethod *pMethod;		// The method which the pin belongs to

	Node(CDrawingObject *,CPoint,CString);	// The constructor for this node
	virtual ~Node();				// The destructor for this node and tree below it
	int Add(NetHash *,Node *);		// Add a node to this tree
	Node *Find(Node *);				// Find a node in this tree
};


// This structure stores the netlist
class Net {
public:
	Net  *next;
	Node *first;
	int  value;

	Net(int,Node *);		// The constructor
	Net() {};				// Another constructor
	virtual ~Net();			// The destructor
};


// The hash table which stores the NetList
class NetHash {
	int size;			// The size of this array
	int CurrentNet;
public:

	Net **array;			// The array which the net is stored in

	int GetNewNet() { return CurrentNet++; }

	NetHash(int);			// The constructor
	virtual ~NetHash();		// The destructor

	Node *GetList(int);		// Get the appropriate net
	void SetList(int,Node *);	// Set the appropriate value
};


// The structure which defines which errors/warnings are in use
struct ErrorTest {
	int DupRef	: 1;		// Duplicated references 0
	int UnConnect	: 1;		// Unconnected items 1
	int NoConnect	: 1;		// Mode than one item on a no-connect net 2
	int Power	: 1;		// Power connected to power 3
	int OutputPwr	: 1;		// Power connected to an output 4
	int Output	: 1;		// Output connected to an output 5
	int NoOutput	: 1;		// No outputs driving inputs 6
	int UnConnected : 1;		// Unconnected nets 7
};

// The type of nets that exist
enum NetType {	nUnknown, 		// The net has not been evaluated yet				0
		nPassive,		// The net only contains passive connections			1
		nInput,			// The net only contains inputs					2
		nOutput,		// The net contains an output and some inputs/passives		3
		nBiDir,			// The net contains a bi-dir/inputs/passives/tri-state		4
		nPower,			// The net contains a power item and some inputs/passives	5
		nNoConnect,		// The net contains a no-connect and one other item		6
		nNoConnectOnly,		// The net contains a no-connect only				7
		nCheck1			// The net needs checking					8
};



// The dialog for the electrical rules check box
class CDlgERCBox : public CDialog {
	ErrorTest 	theErrorTest;

public:	
 	CDlgERCBox(CWnd *pParentWnd = NULL) : CDialog( IDD_ERCBOX, pParentWnd ) {}
	BOOL OnInitDialog();
	virtual void OnOK();

	void SetErrorTest(ErrorTest NewErrorTest) { theErrorTest = NewErrorTest; }
	ErrorTest GetErrorTest() { return theErrorTest; }
};


#endif
