/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "resource.h"
#include "revision.h"
#include "colour.h"
#include "option.h"
#include "registry.h"
#include "layer.h"
#include "diag.h"
#include "symbol.h"



////// The option catorgories //////


Option::Option()
{
	PinLength = 20;
}


void Option::Init( CTinyCadDoc *pDesign )
{
	// Keep a track of the design
	m_pDesign = pDesign;

	// Create the default fonts
	LOGFONT theTextFont;
	
	// Initalise the font settings for normal text
 	theTextFont.lfHeight = sTEXT;
  	theTextFont.lfWidth = 0;	
  	theTextFont.lfEscapement = 0;
  	theTextFont.lfOrientation = 0;
  	theTextFont.lfWeight = 400;
  	theTextFont.lfItalic = FALSE;
  	theTextFont.lfUnderline = FALSE;
  	theTextFont.lfStrikeOut = FALSE;
  	theTextFont.lfCharSet = ANSI_CHARSET;
  	theTextFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
  	theTextFont.lfClipPrecision = CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES;
  	theTextFont.lfQuality = DEFAULT_QUALITY;
  	theTextFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
  	strcpy(theTextFont.lfFaceName,fDEFAULT_FONT);
	theFontList = new ListOfFonts(theTextFont);


	// Initalise the font settings for pin text
 	theTextFont.lfHeight = sPIN;
  	theTextFont.lfWidth = 0;	
  	theTextFont.lfEscapement = 0;
  	theTextFont.lfOrientation = 0;
  	theTextFont.lfWeight = 400;
  	theTextFont.lfItalic = FALSE;
  	theTextFont.lfUnderline = FALSE;
  	theTextFont.lfStrikeOut = FALSE;
  	theTextFont.lfCharSet = ANSI_CHARSET;
  	theTextFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
  	theTextFont.lfClipPrecision = CLIP_DEFAULT_PRECIS  | CLIP_LH_ANGLES;
  	theTextFont.lfQuality = DEFAULT_QUALITY;
  	theTextFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
  	strcpy(theTextFont.lfFaceName,fDEFAULT_FONT);
	theFontList->Add(new ListOfFonts(theTextFont));

	// Initalise the font settings for ruler text
 	theTextFont.lfHeight = sRULE;
  	theTextFont.lfWidth = 0;	
  	theTextFont.lfEscapement = 0;
  	theTextFont.lfOrientation = 0;
  	theTextFont.lfWeight = 400;
  	theTextFont.lfItalic = TRUE;
  	theTextFont.lfUnderline = FALSE;
  	theTextFont.lfStrikeOut = FALSE;
  	theTextFont.lfCharSet = ANSI_CHARSET;
  	theTextFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
  	theTextFont.lfClipPrecision = CLIP_DEFAULT_PRECIS  | CLIP_LH_ANGLES;
  	theTextFont.lfQuality = DEFAULT_QUALITY;
  	theTextFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
  	strcpy(theTextFont.lfFaceName,fDEFAULT_FONT);
	theFontList->Add(new ListOfFonts(theTextFont));


	CurrentFont = 0;


	// Create a the default line thickness
	LineStyle l;

	// The default line style
	l.Colour = cBLACK;
	l.Thickness = 1;
	l.Style = PS_SOLID;
	theStyleList = new ListOfStyles(l);

	// The line style for a dashed line
	l.Colour = cBLACK;
	l.Thickness = 1;
	l.Style = PS_DASH;
	theStyleList->Add(new ListOfStyles(l));

	// The line style for a wire
	l.Colour = cWIRE;
	l.Thickness = 1;
	l.Style = PS_SOLID;
	theStyleList->Add(new ListOfStyles(l));

	// The line style for a bus
	l.Colour = cBUS;
	l.Thickness = 5;
	l.Style = PS_SOLID;
	theStyleList->Add(new ListOfStyles(l));

	CurrentStyle = 0;



	// Some default fill styles
	FillStyle f;
	f.Colour = RGB(0,0,0);	
	f.Index = -1;			// No fill
	theFillStyleList = new ListOfFillStyles(f);

	// Black fill
	f.Colour = RGB(0,0,0);
	f.Index = 0;
	theFillStyleList->Add(new ListOfFillStyles(f));

	// Default - no fill
	CurrentFillStyle = 0;



	// Attempt to load the defaults for the grid
	WORD hold;
	Registry theRegistry;

	GridShow  = theRegistry.Get("ShowGrid",&hold) ? hold : FALSE;
	Units     = theRegistry.Get("Units",&hold) ? hold : 0;
	ScrollBar = theRegistry.Get("ScrollBar",&hold) ? hold : 20;

	PinLength = theRegistry.Get("PinLength", &hold) ? hold : 30;

	
	AutoDrag = theRegistry.Get("AutoDrag",&hold) ? hold : TRUE;
	AutoJunc = theRegistry.Get("AutoJunc",&hold) ? hold : TRUE;
	AutoSnap = theRegistry.Get("AutoSnap",&hold) ? hold : TRUE;
	AutoSnapRange = theRegistry.Get("AutoDragRange",&hold) ? hold : 15;

	theMetaFileList = new ListOfMetaFiles( NULL );


	// Create the no symbol object
	CDrawSquare *Sq = new CDrawSquare(pDesign,xSquareEx3);
	Sq->a=CPoint(0,0);
	Sq->b=CPoint(-40,-40);
	CPoint tr = CPoint(-40,-40);
	drawingCollection method;
	method.push_back( Sq );
	CDesignFileSymbol *NoSymbol = new CDesignFileSymbol( pDesign, method, tr );
	NoSymbol->name = "No Symbol Defined";
	NoSymbol->reference="?";
	NoSymbol->ppp = 1;  
	theSymbolList = new ListOfSymbols( NoSymbol );
}

// The destructor
Option::~Option()
{
	delete theFontList;
	delete theStyleList;
	delete theFillStyleList;
	delete theMetaFileList;
	delete theSymbolList;
}

void Option::UnTag()
{
	theFontList->UnTag();
	theStyleList->UnTag();
	theFillStyleList->UnTag();
	theMetaFileList->UnTag();
	theSymbolList->UnTag();
}



////// Some default settings for dialogs ////// 

void Option::SetPinLength( int pl )
{
	PinLength = pl;
	Registry theRegistry;
	theRegistry.Place("PinLength", PinLength );
}



////// Save the current settings along with the design //////

void Option::SaveOptions( CStream &theArchive )
{
	theArchive << (BYTE)xOptions;
	theArchive << CurrentFont << CurrentStyle << GridShow << Units << ScrollBar;
}

void Option::LoadOptions( CStream &theArchive, BOOL keep )
{
	if (keep)
	{
		theArchive 
			>> CurrentFont >> CurrentStyle >> GridShow >> Units >> ScrollBar;
		CurrentFont = GetNewFontNumber( CurrentFont );
		CurrentStyle = GetNewStyleNumber( CurrentStyle );
	}
	else
	{
		hFONT	dCurrentFont;
		hSTYLE	dCurrentStyle;
		BOOL	dGridShow;
		LONG	dUnits;
		LONG	dScrollBar;
		theArchive 
			>> dCurrentFont >> dCurrentStyle >> dGridShow >> dUnits >> dScrollBar;
	}
}




////// The edit options //////

// Change the size of a font and create a new entry

hFONT Option::ChangeFontSize(hFONT n, int delta_height, int width )
{
  LOGFONT Font = *GetFont( n );
  Font.lfHeight += delta_height;

  if (width != 0)
  {
	Font.lfWidth = width;
  }

  return AddFont( &Font );
}

int Option::GetFontWidth( hFONT n )
{
	LOGFONT *Font = GetFont( n );

  // If we have an aspect ratio that is not 1.0 then we must
  // first find the "average" width of the font.  We can only
  // do this by first selecting the font with an aspect ratio
  // of 1.0 and measuring it.
  if (Font->lfWidth == 0)
  {
	CClientDC dc(AfxGetMainWnd());
	CFont f;
	f.CreateFontIndirect( Font );

	TEXTMETRIC metrics;
	CFont *old_font = dc.SelectObject( &f );
	dc.GetTextMetrics( &metrics );
	dc.SelectObject( old_font );
	Font->lfWidth = metrics.tmAveCharWidth;
  }

  return Font->lfWidth;
}




void Option::ChooseFont(CDC &PrinterDC, HWND Window)
{
	CHOOSEFONT cf;
	LOGFONT aLogFont = *(GetFont(GetCurrentFont()));

	cf.lStructSize = sizeof(CHOOSEFONT);
    cf.hwndOwner = Window;
    cf.hDC = PrinterDC.m_hDC;
    cf.lpLogFont = &aLogFont;
    cf.iPointSize = 10;
    cf.Flags = CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_BOTH;
    cf.rgbColors = RGB(0,0,0);
    cf.lCustData = 0;
    cf.lpfnHook = NULL;
    cf.lpTemplateName = NULL;
    cf.hInstance = NULL;
    cf.lpszStyle = NULL;
    cf.nFontType = SCREEN_FONTTYPE;
    cf.nSizeMin = 5;
    cf.nSizeMax = 128;

	if (::ChooseFont(&cf))
		CurrentFont = AddFont(&aLogFont);
}











////// The symbol operators //////

void Option::ResetSymbols()
{
	ListOfSymbols *pSymbol = theSymbolList;

	while (pSymbol != NULL)
	{
		pSymbol->ResetSymbol();
		pSymbol = static_cast<ListOfSymbols*>(pSymbol->next);
	}
}




// Convert a CPoint to a Unit CString
CString Option::PointToDisplay(CPoint a,BOOL horiz)
{
 	CString r;

  	float Scale = GetUnits()==0 ? (float)PIXELSPERMM : ((float)PIXELSPERMM * 25.4f);
	float sx = a.x / Scale;
	float sy = a.y / Scale;
	if (horiz)
	{
  		sprintf(r.GetBuffer(20),"%.1f",sy,GetUnits()==0 ? "mm" : "\"");
	}
	else
	{
		sprintf(r.GetBuffer(20),"%.1f",sx,GetUnits()==0 ? "mm" : "\"");
	}
	r.ReleaseBuffer();

	return r;
}


// Convert a CPoint to a Unit CString
CString Option::PointToUnit(CPoint a)
{
 	CString r;

  	float Scale = GetUnits()==0 ? (float)PIXELSPERMM : ((float)PIXELSPERMM * 25.4f);
	float sx = a.x / Scale;
	float sy = a.y / Scale;
  	sprintf(r.GetBuffer(20),"%.1f,%.1f%s",sx,sy,GetUnits()==0 ? "mm" : "\"");
	r.ReleaseBuffer();

	return r;
}




