// Stream.h: interface for the CStream class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STREAM_H__B43451E8_657F_401D_A3C9_B34CC6FCB07D__INCLUDED_)
#define AFX_STREAM_H__B43451E8_657F_401D_A3C9_B34CC6FCB07D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CStream
{

public:
	CStream();
	virtual ~CStream();


	// Flush this stream
	virtual void Flush() = 0;

	// Close this stream
	virtual void Close() = 0;

	// The read & write operators
	virtual void Write(const void* lpBuf, UINT nMax ) = 0;
	virtual UINT Read(void* lpBuf, UINT nMax ) = 0;


	////////////////////////////////////////////////////
	// The default operators
	//
	template<class T> CStream &operator<<( const T& a )
	{
		Write( &a, sizeof(T) );
		return *this;
	}

	template<class T> CStream &operator>>( T& a )
	{
		Read( &a, sizeof(T) );
		return *this;
	}

	// The CString operators
	virtual CStream &operator<<( const CString s ) = 0;
	virtual CStream &operator>>( CString &s ) = 0;
};





class CStreamFile : public CStream
{
private:
	CArchive	*m_pArchive;
	BOOL		m_auto_archive;

public:

	CStreamFile( CArchive *a );
	CStreamFile( CFile *f = NULL, UINT nmode = 0);
	virtual ~CStreamFile();


	// Flush this stream
	virtual void Flush()
	{
		m_pArchive->Flush();
	}

	// Close this stream
	virtual void Close()
	{
		m_pArchive->Close();
	}

	// Attach a file
	void Attach( CFile *theFile, UINT nmode )
	{
		m_pArchive = new CArchive( theFile, nmode );
		m_auto_archive = TRUE;
	}

	// Get the file associated with this stream
	CFile* GetFile()
	{
		return m_pArchive->GetFile();
	}



	// The write operator
	virtual void Write(const void* lpBuf, UINT nMax )
	{
		m_pArchive->Write( lpBuf, nMax );
	}

	virtual UINT Read(void* lpBuf, UINT nMax )
	{
		return m_pArchive->Read( lpBuf, nMax );
	}

	////////////////////////////////////////////////////
	// The default operators
	//
	template<class T> CStream &operator<<( const T& a )
	{
		Write( &a, sizeof(T) );
		return *this;
	}

	template<class T> CStream &operator>>( T& a )
	{
		Read( &a, sizeof(T) );
		return *this;
	}


	////////////////////////////////////////////////////
	// The CString operator
	//
	virtual CStream &operator<<( const CString s )
	{
		(*m_pArchive) << s;
		return *this;
	}

	virtual CStream &operator>>( CString &s )
	{
		(*m_pArchive) >> s;
		return *this;
	}


};





#endif // !defined(AFX_STREAM_H__B43451E8_657F_401D_A3C9_B34CC6FCB07D__INCLUDED_)
