/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if !defined(AFX_TINYCADVIEW_H__A2D9E56B_4BF2_4F22_98DB_450C36940721__INCLUDED_)
#define AFX_TINYCADVIEW_H__A2D9E56B_4BF2_4F22_98DB_450C36940721__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "resource.h"
#include "option.h"
#include "context.h"
#include "ruler.h"
#include "object.h"
#include "library.h"
#include "TinyCadDoc.h"
#include "diag.h"
#include "EditToolBar.h"

#define STRLEN	256

class Centre;
class CDlgPositionBox;

extern CEditToolbar g_EditToolBar;


class CTinyCadView : public CView
{
protected: // create from serialization only
	CTinyCadView();
	DECLARE_DYNCREATE(CTinyCadView)

	// Have we captured the mouse control?
	BOOL	m_captured;

	// Are we panning with the middle mouse button
	BOOL	m_panning;


	UINT	ClipboardFormat;		// The registered clipboard format

	/////////////////////////////////////////////////////////////////////////////
	//

	CStatusBar m_wndStatusBar;	// The status bar associated with this window

	CPoint 	MousePosition;		// The last known mouse position

	// Change the current offset co-ords
	friend Centre;
	void SetScroll(int,int);
	// Set a new zoom value
	void ChangeZoom(LONG);
	void SetZoom(LONG);

	// The rulers
	Ruler *vRuler;
	Ruler *hRuler;

	Transform theTransform;			// The transform of the design to the window
	Transform current_Transform;	// The cache of the transform whilst printing...


	// The printing information
	int	m_rows;
	int m_cols;
	BOOL m_Fit;

	HCURSOR m_mouse_pointers[13];


// Attributes
public:
	CTinyCadDoc* GetDocument();

protected:
	// Our own version of this function, so that we
	// can set portrait/landscape mode automatically
	BOOL DoPreparePrinting(CPrintInfo* pInfo);

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTinyCadView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	virtual void OnInitialUpdate();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CTinyCadView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

	void SetScrollCentre(CPoint);


	// Read and write the registry defaults
	void RegistryInit();
	void RegistryRead();

 	// Get a symbol from the libraries
	void SetLibEdit(CLibraryStore *NewLibEdit);
	
	// Clipboard operators
	afx_msg void OnDestroyClipboard();
	BOOL IsClipboardAvailable();


// Generated message map functions
protected:
	//{{AFX_MSG(CTinyCadView)
	afx_msg void OnUpdateEditedit(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBusback(CCmdUI* pCmdUI);
	afx_msg void OnUpdateBusslash(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolarc(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolbus(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolbusname(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolcircle(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolconnect(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolget(CCmdUI* pCmdUI);
	afx_msg void OnUpdateTooljunc(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToollabel(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolpolygon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolpower(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolsquare(CCmdUI* pCmdUI);
	afx_msg void OnUpdateTooltext(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolwire(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewcentre(CCmdUI* pCmdUI);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnUpdateEditdrag(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditdup(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditrotate(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditmove(CCmdUI* pCmdUI);
	afx_msg void OnSnaptogrid();
	afx_msg void OnUpdateSnaptogrid(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatdown(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatleft(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatnamedown(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatnameup(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatpindown(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatpinup(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatright(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRepeatup(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditpaste(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditcut(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditcopy(CCmdUI* pCmdUI);
	afx_msg void OnDestroy();
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnLibraryRefreshsymbolsfromlibraries();
	afx_msg void OnRulerVert();
	afx_msg void OnUpdateRulerVert(CCmdUI* pCmdUI);
	afx_msg void OnRulerHoriz();
	afx_msg void OnUpdateRulerHoriz(CCmdUI* pCmdUI);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnViewOptions();
	afx_msg void OnContextArcin();
	afx_msg void OnContextArcout();
	afx_msg void OnContextCanceldrawing();
	afx_msg void OnContextFinishdrawing();
	afx_msg void OnContextFreeline();
	afx_msg void OnContextAddhandle();
	afx_msg void OnContextDeletehandle();
	afx_msg void OnContextZorderBringtofront();
	afx_msg void OnContextZorderSendtoback();
	afx_msg void OnUpdateContextZorderBringtofront(CCmdUI* pCmdUI);
	afx_msg void OnUpdateContextZorderSendtoback(CCmdUI* pCmdUI);
	//}}AFX_MSG

	afx_msg int	 OnCreate( LPCREATESTRUCT );

	// The Help Menu
	afx_msg void OnAbout();

	// The File Menu
	afx_msg void OnFileDesign();
	afx_msg void OnFileImport();
	afx_msg void OnFileExport() { GetDocument()->SelectObject(new CDrawBlockExport(GetDocument())); }
	afx_msg void OnFilePageSet();


	// The Edit Menu
	afx_msg void OnEditUndo() { GetDocument()->SelectObject(new CDrawEditItem(GetDocument())); GetDocument()->Undo(); }
	afx_msg void OnEditRedo() { GetDocument()->SelectObject(new CDrawEditItem(GetDocument())); GetDocument()->Redo(); }
	afx_msg void OnEditRepeat();
	afx_msg void OnEditLayer();
	afx_msg void OnEditEdit() { GetDocument()->SelectObject(new CDrawEditItem(GetDocument())); }
	afx_msg void OnEditDelete();
	afx_msg void OnEditMove() { GetDocument()->SelectObject(new CDrawBlockMove(GetDocument())); }
	afx_msg void OnEditDrag() { GetDocument()->SelectObject(new CDrawBlockDrag(GetDocument())); }
	afx_msg void OnEditDup() { GetDocument()->SelectObject(new CDrawBlockDup(GetDocument())); }
	afx_msg void OnEditRotate() { GetDocument()->SelectObject(new CDrawBlockRotate(GetDocument())); }
	afx_msg void OnFindFind();
	afx_msg void OnFindSetTag();
	afx_msg void OnFindJumpTag();
	afx_msg void OnEditCopy();
	afx_msg void OnEditCut();
	afx_msg void OnEditPaste();


	// The Keyboard operations
	afx_msg void OnViewZoomIn();
	afx_msg void OnViewZoomOut();
	afx_msg void OnViewCentreX() { SetScrollCentre(MousePosition); }

	// The scrollbar/sizing operations
	afx_msg void OnHScroll(UINT,UINT,CScrollBar *);
	afx_msg void OnVScroll(UINT,UINT,CScrollBar *);

   	// The Toolbar Menu
	afx_msg void OnSelectWire() { GetDocument()->SelectObject(new CDrawLine(GetDocument(),xWire)); }
	afx_msg void OnSelectBus() { GetDocument()->SelectObject(new CDrawLine(GetDocument(),xBus)); }
	afx_msg void OnSelectBusSlash() { GetDocument()->SelectObject(new CDrawBusSlash(GetDocument(),0)); }
	afx_msg void OnSelectBusBack() { GetDocument()->SelectObject(new CDrawBusSlash(GetDocument(),1)); }
	afx_msg void OnSelectBusName() { GetDocument()->SelectObject(new CDrawText(GetDocument(),xBusNameEx)); }
	afx_msg void OnSelectJunction();
	afx_msg void OnSelectPower() { GetDocument()->SelectObject(new CDrawPower(GetDocument())); }
	afx_msg void OnSelectLabel() { GetDocument()->SelectObject(new CDrawText(GetDocument(),xLabelEx2)); }
	afx_msg void OnSelectConnect() { GetDocument()->SelectObject(new CDrawNoConnect(GetDocument())); }
	afx_msg void OnSelectPolygon();
	afx_msg void OnSelectArc();
	afx_msg void OnSelectSquare() { GetDocument()->SelectObject(new CDrawSquare(GetDocument(),xSquareEx3)); }
	afx_msg void OnSelectCircle() { GetDocument()->SelectObject(new CDrawSquare(GetDocument(),xCircleEx3)); }
	afx_msg void OnSelectText() { GetDocument()->SelectObject(new CDrawText(GetDocument(),xTextEx2)); }
	afx_msg void OnViewCentre() { GetDocument()->SelectObject(new CDrawCentre(GetDocument())); }
	afx_msg void OnSelectGet();

	// The Special Menu
	afx_msg void OnSpecialAnotate();
	afx_msg void OnSpecialBom();
	afx_msg void OnSpecialNet();
	afx_msg void OnSpecialCheck();

	// The Repeat Menu
	afx_msg void OnRepeatUp() { GetDocument()->SetRepeatDir(0); }
	afx_msg void OnRepeatDown() { GetDocument()->SetRepeatDir(1); }
	afx_msg void OnRepeatLeft() { GetDocument()->SetRepeatDir(2); }
	afx_msg void OnRepeatRight() { GetDocument()->SetRepeatDir(3); }
	afx_msg void OnRepeatNameUp();
	afx_msg void OnRepeatNameDown();
	afx_msg void OnRepeatPinUp();
	afx_msg void OnRepeatPinDown();


	// Tracking mouse movements
	afx_msg void OnMouseMove(UINT nFlags, CPoint p);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint p);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint p);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint p);
	afx_msg void OnMButtonDown( UINT nFlags, CPoint point );
	afx_msg void OnMButtonUp( UINT nFlags, CPoint point );


	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in TinyCadView.cpp
inline CTinyCadDoc* CTinyCadView::GetDocument()
   { return (CTinyCadDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TINYCADVIEW_H__A2D9E56B_4BF2_4F22_98DB_450C36940721__INCLUDED_)
